/*
   joystick throttle driver - mostly based on the joystick test driver
*/

static char rcsid[] = "$Id: jthrot.c,v 1.2 1997/05/26 10:18:00 bri Exp $";

#include <linux/joystick.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define E_DELTA 7
#define MS_DELAY 100

#ifndef USE_PRINTF
int print_num(int n, int what, FILE *F);
int print_str(char *a_str, FILE *F);
#endif

int main (int argc, char **argv)
{
	int fd, status;
	char *fname;
	struct JS_DATA_TYPE js, last;

	/* should be one argument, and it should be "0" or "1" */
	if (argc != 2 || (strcmp (argv[1], "0") && strcmp (argv[1], "1"))) {
#ifndef USE_PRINTF
		print_str("usage: js <num>\n", stderr);
#else
		fprintf (stderr, "usage: js <num>\n");
#endif
		exit (1);
	}

	/* pick appropriate device file */
	if (!strcmp (argv[1], "0"))
		fname = "/dev/js0";
	else if (!strcmp (argv[1], "1"))
		fname = "/dev/js1";
	else
		fname = NULL;

	/* open device file */
	fd = open (fname, O_RDONLY);
	if (fd < 0) {
#ifndef USE_PRINTF
		print_str("no joystick\n", stdout);
#else
		fprintf(stdout,"no joystick\n");
#endif
		exit (0);
	}

/*	printf ("Joystick test program (interrupt to exit)\n"); */

	/* this is fairly dumb, but it'll work */

	last.buttons = 0;
	last.x = -1000;
	last.y = 0;

	while (1) {
		status = read (fd, &js, JS_RETURN);
		if (status != JS_RETURN) {
			perror (argv[0]);
			exit (1);
		}

		if ((abs(last.x - js.x) > E_DELTA) ||
		    (abs(last.y - js.y) > E_DELTA) ||
		    (last.buttons != js.buttons)) {
#ifndef USE_PRINTF
		    print_num(js.x, 1, stdout);
		    print_num(js.y, 1, stdout);
		    print_num(js.buttons & 1, 1, stdout);
		    print_num((js.buttons >> 1) & 1, 2, stdout);
#else
		    fprintf(stdout, "%d %d %d %d\r\n", js.x, js.y,
			    js.buttons & 1, (js.buttons >> 1) & 1);
#endif
		    last.x = js.x;
		    last.y = js.y;
		    last.buttons = js.buttons;
		    fflush (stdout);
		}

		/* sleep until next poll */
		usleep (MS_DELAY);
	}

	exit (0);
}

#define BS 60

#ifndef USE_PRINTF

/* avoid using fprintf */

int print_num(int n, int what, FILE *F) {
    char buf[BS], *p;
    int divider;
    size_t i;

    int zero = (int) '0';

    i = 1;
    p = &buf[BS - 3];

    while ((n / 10) != 0) {
	*p = (char) (n % 10) + zero;
	n = n / 10;
	p--; i++;
    }
    *p = (char) (n % 10) + zero;

    /* XXX ascii */
    if (what == 1) {
	i++; buf[BS - 2] = ' ';
    } else if (what == 2) {
	i += 2;
	buf[BS - 2] = '\r'; buf[BS - 1] = '\n';
    }
    fwrite (p, i, 1, F);
    return (i);
}

int print_str(char *a_str, FILE *F) {
    int i;

    fwrite(a_str, i = strlen(a_str), 1, F);
    return (i);
}

#endif	/* USE_PRINTF */

