/* Lenz protocol */

static char rcsid[] = "$Id: lenz.c,v 1.5 1997/05/26 16:48:12 bri Exp $";

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>

#include <stdio.h>

#include "common.h"
#include "lenz.h"

unsigned char output_buf[30];	/* we'll use this periodically */

/* take the fd and read forever from it */
int do_reading(int FD) {
    unsigned char buf[30];
    int l;
    while (1) {	/* perhaps should mash this together.. */
	l = read_packet(FD, buf);
	interpret_packet(buf, l);
    }
}

int init_line(char *name) {
    int FD;
    struct termios interface;

    if ((FD=open(name,O_RDWR)) == -1) {
	fprintf(stderr,"dammit, couldn't open device.\n");
	return(-1);
    }
    tcgetattr(FD, &interface);
    interface.c_oflag = ONOCR | ONLRET;
    interface.c_oflag &= ~(OLCUC| ONLCR | OCRNL);
    interface.c_cflag = CS8 | CRTSCTS ;
    interface.c_cflag &= ~(CSTOPB | PARENB);
    interface.c_iflag = IGNCR | IGNBRK | IGNPAR;
    interface.c_iflag &= ~(ISTRIP | IXON | IXOFF );
    interface.c_lflag = NOFLSH | IEXTEN;
    interface.c_lflag &= ~(ISIG | ICANON | ECHO | TOSTOP | PENDIN);

    cfsetospeed(&interface, B9600);

    tcsetattr(FD, TCSAFLUSH, &interface);
    return(FD);
}

/* read_packet takes a file descriptor and a buffer, puts the packet in the
   buffer, and returns the length of the packet. -1 means an invalid
   packet.

   As long as the buffer is over 15 chars long, there won't be any
   problems; the protocol can't handle anything longer than 15. Thus, you
   probably want to use a static buffer with this, because it won't really
   take up much of any memory, and will save you in syscalls.
   */
int read_packet(int FD, unsigned char *buffer) {
    int i, len;
    unsigned char checksum;
    unsigned char *front;

    i = 0;
    front = buffer;
    *buffer = readbyte(FD);
    len = (int) *buffer & 0x0f;
#ifdef DEBUG
    fprintf (stderr, " header top: %x, number of bytes %d\n",
	*buffer & 0xf0, len);
#endif
    while (i < len) {
	i++;
	buffer++;
	*buffer = readbyte(FD);
    }
    checksum = readbyte(FD);
    if (checksum != mkchecksum(front, len)) {
#ifdef DEBUG
	fprintf(stderr, ":checksum %x %x wrong:\n", checksum,
	    mkchecksum(front, len));
#endif
	return (-1);
    }
    return (i);
}

/* the Lenz checksum is an xor of the header and data bytes.
   important: as "length", give this function the number of data bytes! */
unsigned char mkchecksum(unsigned char *header, int length) {
    unsigned char xor_so_far;
    int i;

    i = 0;
    xor_so_far = *header;
    while (i < length) {
	header++;
	i++;
	xor_so_far = *header ^ xor_so_far;
    }
    return(xor_so_far);
}

unsigned char readbyte(int FD) {
    int i;
    unsigned char the_byte;
    i = read(FD, &the_byte, 1);
#ifdef DEBUG
    if (the_byte) {
	fprintf(stderr, ":%x, %d\n", the_byte & 0xff, the_byte & 0xff);
    }
#endif
    return (the_byte);
}

/* figure out what the various packets mean, stdout them, maybe do
   something else */
int interpret_packet(unsigned char *buffer, int length) {
    int other_controller, loco_addr, direction, speed, functions, top_speed;

    other_controller = 0;

    switch(*buffer) {

	/* reaction to a command */
	case H_ANSWER:	
	    buffer++;
	    switch (*buffer) {
		case A_OKAY:
		fprintf(stdout, "ok\n");	/* okay, command good */
		break;
		case ERR_UNKNOWN:
		fprintf(stdout, "un\n");	/* unknown command */
		break;
		case ERR_INT_PC:
		fprintf(stdout, "er ip\n");	/* error interface-pc */
		break;
		case ERR_INT_COM:
		fprintf(stdout, "er ic\n");	/* error interface-command
						    station */
	    }
	    break;

	/* some sort of information about DCC system */
	case H_INFO:
	    buffer++;
	    switch(*buffer) {
		case H_INFO_VER:
		    buffer++;
		    fprintf(stdout, "vr Lenz LZ100 version %d.%d\n",
			(*buffer & 0xf0) >> 4, *buffer & 0x0f);
		case H_INFO_STAT:
		    buffer++;
		    fprintf(stdout, "st %d\n", *buffer & 0x0f);
	    }
	    break;

	/* loco has been taken over by some other controller */
	case H_OTHER_CON:
	    buffer++;
	    fprintf(stdout, "oc %d\n", *buffer);
	    break;

	/* locomotive information */
	case H_LOCO_INFO_OC:
	    other_controller = 1;
	case H_LOCO_INFO:
	    buffer++;
	    loco_addr = (int) *buffer;
	    buffer++;
	    direction = (int) ((*buffer >> 6) & 0x01);	/* 6th bit */
	    functions = (int) ((*buffer >> 5) & 0x01);	/* F0 is bit 5 */
	    speed = (int) (*buffer & 0x1f);	/* we mess with this later */
	    buffer++;
	    functions |= (int) (*buffer << 1) & 0x1e; /* F1-F4 */
	    buffer++;
	    if (*buffer & 0x03 ) {	/* 27- or 28-speed mode */
		if (*buffer & 0x01) {
		    top_speed = 27;
		} else {
		    top_speed = 28;
		}
		if (speed & 0x1f) {
		    speed = ((speed & 0x0f) << 1) |
			    ((speed & 0x10) >> 4);
		    if (speed == 2) {
			speed = -1;		/* emergency stop */
		    } else {
			speed -= 3;		/* wacky, eh? */
		    }
		} else {
		    speed = 0;
		}
	    } else {				/* 14-speed mode */
		top_speed = 14;
		if (speed & 0x0f) {
		    speed = (int) (speed & 0x0f);
		    speed--;
		    if (!speed) {
			speed = -1; 		/* emergency stop */
		    }
		} else {
		    speed = 0;
		}
	    }
	    fprintf(stdout, "li %d %d %d %d %d %d\n",
		loco_addr, direction, speed, functions, top_speed,
		other_controller);
	    break;

	/* programming info */
	case H_CV_INFO:
	    buffer++;
	    switch (*buffer) {
		case READ_OK_ALT:		/* huh? */
		case READ_OK:			/* we have info */
		    buffer++;
		    loco_addr = (int) *buffer;	/* CV number */
		    buffer++;
		    speed = (int) *buffer;	/* CV value */
		    fprintf(stdout, "cv %d %d\n", loco_addr, speed);
		    break;
		case NO_DECODER:	/* no decoder */
		    fprintf(stdout, "pe nd\n");
		    break;
		case SHORT_CIRCUIT:	/* short */
		    fprintf(stdout, "pe sc\n");
		    break;
		case PROG_BUSY:		/* someone else is using it */
		    fprintf(stdout, "pe bu\n");
		    break;
		default:
		    fprintf(stdout, "pe %d\n", *buffer);
		    break;
	    }
	    break;

	case H_MODE_INFO:		/* some misc info */
	    buffer++;
	    switch (*buffer) {
		case M_POWER_OFF:
		    fprintf(stdout, "mi power_off\n");
		    break;
		case M_POWER_ON:
		    fprintf(stdout, "mi go\n");
		    break;
		case M_PROG:
		    fprintf(stdout, "mi prog_mode\n");
		    break;
		case P_NO_DECODER:
		    fprintf(stdout, "pe nd\n");
		    break;
		case P_SHORT_CIRCUIT:	/* short */
		    fprintf(stdout, "pe sc\n");
		    break;
		case DT_NO_COMMAND:
		    fprintf(stdout, "de no_command\n");
		    break;
		case DT_BUSY:
		    fprintf(stdout, "de dt_busy\n");
		    break;
		case DT_LOK_BUSY:
		    fprintf(stdout, "de loco_busy\n");
		    break;
		case DT_LOK_MOVE:
		    fprintf(stdout, "de loco_moving\n");
		    break;
		default:
		    fprintf(stdout, "mi %d\n", *buffer);
		    break;
	    }
	    break;

	default:			/* we dunno */
	    /* we use loco_addr as a counter */
	    loco_addr = (int) (*buffer & 0x0f);
	    fwrite("ul", 3, 1, stdout);
	    while (loco_addr >= 0) {
		fprintf(stdout, " %x", *buffer);
		buffer++;
		loco_addr--;
	    }
	    fprintf(stdout, "\n");
    }
    fflush(stdout);
    return (0);
}

int get_version(int OFD) {
    unsigned char *a, *b;
    a = output_buf;
    *a = H_GET_INFO;	a++; 
    *a = GET_VERSION;	a++;
    *a = mkchecksum(output_buf, 1);
    write(OFD, output_buf, 3);
    return (0);
}

int get_status(int OFD) {
    unsigned char *a, *b;
    a = output_buf;
    *a = H_GET_INFO;	a++; 
    *a = GET_STATUS;	a++;
    *a = mkchecksum(output_buf, 1);
    write(OFD, output_buf, 3);
    return (0);
}

int power_on(int OFD) {
    unsigned char *a;
    a = output_buf;
    *a = H_MODES;	a++; 
    *a = TURN_ON;	a++;
    *a = mkchecksum(output_buf, 1);
    write(OFD, output_buf, 3);
    return (0);
}

int power_off(int OFD) {
    unsigned char *a;
    a = output_buf;
    *a = H_MODES;	a++; 
    *a = TURN_OFF;	a++;
    *a = mkchecksum(output_buf, 1);
    write(OFD, output_buf, 3);
    return (0);
}

int emerg_stop(int OFD) {
    unsigned char *a;
    a = output_buf;
    *a = H_EMERG_STOP;	a++; 
    *a = mkchecksum(output_buf, 0);
    write(OFD, output_buf, 2);
    return (0);
}

int lenz_read_cv(int OFD, int cv) {
    unsigned char *a;
    a = output_buf;
    *a = H_READ_CV;	a++; 
    *a = READ_CV;	a++;
    *a = (unsigned char) cv;	a++;
    *a = mkchecksum(output_buf, 2);
    write(OFD, output_buf, 4);
    return (0);
}

int lenz_answer_me(int OFD) {
    unsigned char *a;
    a = output_buf;
    *a = H_PROG_ANSWER;	a++; 
    *a = PROG_ANSWER;	a++;
    *a = mkchecksum(output_buf, 1);
    write(OFD, output_buf, 3);
    return (0);
}

int get_loco_info(int OFD, int loco) {
    unsigned char *a, *b;

    /* XXX - I have NO IDEA why, but my li100 barfs on 13? */
    if (loco == 13) {
	return(0);
    }
    a = output_buf;
    *a = H_GET_LOCO_INFO;		a++; 
    *a = (unsigned char) loco;		a++;
    *a = (unsigned char) 0;		a++;	/* we don't want to know the mode */
    *a = mkchecksum(output_buf, 2);
    write(OFD, output_buf, 4);
    return (0);
}

int control_loco(int OFD, int loco_addr, int direction, 
                int speed, int functions, int top_speed) {
    unsigned char *a, *b;
    a = output_buf;
    *a = H_CONTROL_LOCO;		a++;
    *a = (unsigned char) loco_addr;	a++;
    *a = (unsigned char) ((direction & 0x01) << 6);	/* direction */
    *a |= (unsigned char) ((functions & 0x01) << 5);	/* F0 */

    if (speed == 0) {		/* most bets are off if speed == 0 */
	*a &= (unsigned char) SPEED_0_MASK;
    } else {
	if (top_speed == 27 || top_speed == 28) {
	    speed += 3;
	    /* bit 4 is least significant bit, sigh.. */
	    *a |= (unsigned char) (((speed >> 1) & 0x0f) |
				   ((speed << 4) & 0x10));
	} else {
	    speed += 1;
	    *a |= (unsigned char) (speed & 0x1f);
	}
    }

    a++;
    *a = (unsigned char) ((functions >> 1) & 0x0f);	/* F1-F4 */

    a++;
    /* ModSel - how many speed steps */
    /* could be a little more efficient here.. */

    switch (top_speed) {
	case 28:
	*a = (unsigned char) MODSEL_28;		/* 28-speed mode */
	break;

	case 27:
	*a = (unsigned char) MODSEL_27;		/* 27-speed mode */
	break;

	default:			/* 14 or something wacky */
	*a = (unsigned char) MODSEL_14;
	break;
    }

    a++;
    *a = mkchecksum(output_buf, 4);
    write(OFD, output_buf, 6);
    return (0);
}

int control_acc(int OFD, int acc, int ac_fun, int is_on) {
    unsigned char *a;
    int b;

    a = output_buf;
    *a = H_ACC;
    a++;

    *a = (unsigned char) ((acc >> 2) & 0x3f);
	/* the accessory should be in the range 0-255 */
	/* they are in groups of 4 */
    a++;

    *a = (unsigned char) 0x80;		/* bit 7 must always be set */
    *a |= (unsigned char) ((is_on << 3) & 0x08);	/* is it on? */
    *a |= (unsigned char) ((acc << 1) & 0x06);
			/* which acc in group? */
    *a |= (unsigned char) (ac_fun & 0x01);	/* which direction? */

    a++;

    *a = mkchecksum(output_buf, 2);
    write(OFD, output_buf, 4);
    return (0);
}

