/* header for lenz protocol */

/*
    $Id: lenz.h,v 1.4 1997/05/04 20:04:15 bri Exp $
*/

#ifndef LENZ_H
#define LENZ_H

/* prototypes */

unsigned char readbyte(int FD);
int read_packet(int FD, unsigned char *buffer);
unsigned char mkchecksum(unsigned char *header, int length);
int interpret_packet(unsigned char *buffer, int length);
int write_packet(int FD, unsigned char *buffer, int length);

/* Lenz LI100 protocol*/

/* headers and other identification info */

#define H_INFO			0x62
#define  H_INFO_VER		0x21
#define  H_INFO_STAT		0x22

#define H_LOCO_INFO		0x84

#define H_LOCO_INFO_OC		0xa4

#define H_ANSWER		0x01

#define  ERR_INT_PC		0x01
#define  ERR_INT_COM		0x02
#define  ERR_UNKNOWN		0x03
#define  A_OKAY			0x04
#define  ERR_COM_DEAD		0x05

#define H_OTHER_CON		0xa3

#define H_MODE_INFO		0x61
#define  M_POWER_OFF		0x00
#define  M_POWER_ON		0x01
#define  M_PROG			0x02
#define  P_NO_DECODER		0x12
#define  P_SHORT_CIRCUIT	0x13
#define  DT_NO_COMMAND		0x83
#define  DT_BUSY		0x84
#define  DT_LOK_BUSY		0x85
#define  DT_LOK_MOVE		0x86

#define H_CV_INFO		0x63
#define  READ_OK		0x11
#define  READ_OK_ALT		0x10
#define  NO_DECODER		0x12
#define  SHORT_CIRCUIT		0x13
#define  PROG_BUSY		0x1f

/* commands */

#define H_GET_INFO		0x21	/* various info */
#define  GET_VERSION		0x21	/* data */
#define  GET_STATUS		0x24	/* data */

#define H_GET_LOCO_INFO		0xa2	/* get loco info */

#define H_CONTROL_LOCO		0xb4	/* control loco */

#define H_CONTROL_LOCO_NMS	0xb3	/* without ModSel, not used */
#define  MODSEL_14		0x00	/* 14-speed mode */
#define  MODSEL_27		0x01	/* 27-speed mode */
#define  MODSEL_28		0x02	/* 28-speed mode */
#define  SPEED_0_MASK		0xe0	/* speed 0 */

#define H_ACC			0x52	/* accessory control */

#define H_MODES			0x21	/* change the system mode */
#define  TURN_OFF		0x80	/* turn track power off */
#define  TURN_ON		0x81	/* turn track power on */

#define H_EMERG_STOP		0x80	/* emergency stop */

/* programming mode commands */
#define H_READ_CV		0x22	/* read a cv */
#define  READ_CV		0x11	/* "" */
#define H_PROG_ANSWER		0x21	/* ask command station to answer */
#define  PROG_ANSWER		0x10	/* "" */

#endif
