/* test the interface/serial port */

static char rcsid[] = "$Id: mtd.c,v 1.3 1997/05/04 20:00:50 bri Exp $";

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>

#include "common.h"

pid_t kid = 0;	/* for the reader */
int FD;		/* file descript. for interface (serial port, ..) */

main() {
    unsigned char a, b, x, buf[30], buf2[5], *p;

    if ((FD = init_line("/dev/cua0")) == -1) {
	exit (1);
    }

    switch(kid = fork()) {
	case -1:
	    fprintf(stderr,"no fork.\n");	/* bzzt */
	    exit(2);
	case 0:			/* child (reader) */
	    do_reading(FD);	/* should never return */
	default: 		/* parent (writer) */
	    break;
    }


    parse_input(FD);

    clean_up_and_exit();
}

int clean_up_and_exit() {
    if(kid) {
	kill(kid, SIGTERM);
	/* perhaps wait() here later */
    }
    close(FD);
    exit(0);
}

/* input parser */
int parse_input(int OFD) {
    /* take stdin for now */
    char i_buf[128], a, b, x, *c;
    int i, j, k, l, m;
    while (fgets(i_buf,127,stdin)) {
	/* we try to go in order of the most frequently used commands */

	/* command a loco */
	if (! str2cmp(i_buf, "lc")) {
	    sscanf(i_buf, "lc %d %d %d %d %d", &i, &j, &k, &l, &m);
	    if (i) control_loco(OFD, i, j, k, l, m);
	    continue;
	}

	/* get loco info */
	if (! str2cmp(i_buf, "li")) {
	    sscanf(i_buf, "li %d", &i);
	    if (i) get_loco_info(OFD, i);
	    continue;
	}

	/* control an accessory */
	if (! str2cmp(i_buf, "ac")) {
	    sscanf(i_buf, "ac %d %d %d", &i, &j, &k);
	    control_acc(OFD, i, j, k);
	    continue;
	}

	/* get DCC system version */
	if (! str2cmp(i_buf, "vr")) {
	    get_version(OFD);
	    continue;
	}

	/* get DCC system status */
	if (! str2cmp(i_buf, "st")) {
	    get_status(OFD);
	    continue;
	}

	if (! str2cmp(i_buf, "go")) {
	    power_on(OFD);
	    continue;
	}

	if (! str2cmp(i_buf, "po")) {
	    power_off(OFD);
	    continue;
	}

	if (! str2cmp(i_buf, "es")) {
	    emerg_stop(OFD);
	    continue;
	}

	if (! str2cmp(i_buf, "lr")) {
	    sscanf(i_buf, "lr %d", &i);
	    lenz_read_cv(OFD, i);
	    continue;
	}

	if (! str2cmp(i_buf, "la")) {
	    lenz_answer_me(OFD);
	    continue;
	}
    }
}

/* slightly less general version of strncmp() */
int str2cmp(char *a, char *b) {
    if (*a == *b) {
	a++; b++; if (*a == *b) return (0);
    }
    return (1);
}

