#!/usr/local/bin/wish
#
# $Id: tklayout.tcl,v 1.4 1997/05/13 18:47:57 bri Exp $
#

set offset_x 0
set offset_y -20

set lay_x 430
set lay_y 350

set l_outer_w 7
set l_inner_w [expr $l_outer_w - 2]
set scale 0.22

set track_color lightblue
set inactive_color darkred
set outline_color black
set ac_track_color darkgreen
set ac_active_color red

proc mklcan {w} {
    global lay_x lay_y
    catch {destroy $w}
    toplevel $w
    canvas $w.c -height $lay_y -width $lay_x
    pack $w.c
}

proc rot_x {x ang} {
    global pi
    return [expr cos($pi * $ang / 180.0) * $x]
}

proc rot_y {y ang} {
    global pi
    return [expr sin($pi * $ang / 180.0) * $y]
}

proc conv_y {y} {
    global scale lay_y offset_y
    return [expr $lay_y - $scale * $y - $offset_y]
}

proc conv_x {x} {
    global scale offset_x
    return [expr $offset_x + $scale * $x]
}

proc mk_str {w name x y ang len} {
    global l_outer_w l_inner_w track_color outline_color
    $w create line [conv_x $x] [conv_y $y] \
	[conv_x [expr $x + [rot_x $len $ang]]] \
	[conv_y [expr $y + [rot_y $len $ang]]] \
	-width $l_outer_w -fill $outline_color -tags "$name $name.strbot"
    $w create line [conv_x $x] [conv_y $y] \
	[conv_x [expr $x + [rot_x $len $ang]]] \
	[conv_y [expr $y + [rot_y $len $ang]]] \
	-width $l_inner_w -fill $track_color -tags "$name $name.strtop"
}

proc mk_curv {w name x y ang rad len} {
    global l_outer_w l_inner_w pi pi_2 offset_x offset_y track_color \
    outline_color
    set cc_x [expr $x + [rot_x $rad [expr 90.0 + $ang]]]
    set cc_y [expr $y + [rot_y $rad [expr 90.0 + $ang]]]
    set cb_x [conv_x [expr $cc_x - $rad]]
    set ce_x [conv_x [expr $cc_x + $rad]]
    set cb_y [conv_y [expr $cc_y - $rad]]
    set ce_y [conv_y [expr $cc_y + $rad]]
    $w create arc $cb_x $cb_y $ce_x $ce_y -style arc \
	-start [expr $ang - 90.0] \
	-extent $len \
	-width $l_outer_w -outline $outline_color -tags "$name $name.curbot"
    $w create arc $cb_x $cb_y $ce_x $ce_y -style arc \
	-start [expr $ang - 90.0] \
	-extent [expr $len] \
	-width $l_inner_w -outline $track_color -tags "$name $name.curtop"
}

proc mk_turnout {w name x y ang len rad deg left_right} {
    global pi
    mk_str $w $name $x $y $ang $len
    if {$left_right == "right"} {
	mk_curv $w $name $x $y [expr 180.0 + $ang] $rad [expr -1 * $deg]
    } else {
	mk_curv $w $name $x $y $ang $rad $deg
    }
}

proc config_turnout {w tnum addr} {
    global turnout
    $w bind $tnum <1> "swap_turnout %W $tnum $addr"
    set turnout($tnum) 1
    update_turnout_display $w $tnum
}

proc operate_turnout {tnum addr on_off} {
    global turnout FD
    puts $FD "ac $addr $turnout($tnum) $on_off"
    flush $FD
}

proc swap_turnout {w tnum addr} {
    global turnout
    set turnout($tnum) [expr ! $turnout($tnum)]
    update_turnout_display $w $tnum
    operate_turnout $tnum $addr 1
    after 200 "operate_turnout $tnum $addr 0"
}

proc update_turnout_display {w tnum} {
    global turnout track_color inactive_color outline_color
    if {$turnout($tnum)} {
	$w itemconfig $tnum.curtop -outline $track_color
	$w itemconfig $tnum.strtop -fill $inactive_color
	$w raise $tnum.curbot
	$w raise $tnum.curtop
    } else {
	$w itemconfig $tnum.curtop -outline $inactive_color
	$w itemconfig $tnum.strtop -fill $track_color
	$w raise $tnum.strbot
	$w raise $tnum.strtop
    }
}

proc update_uncoupler_display {w tnum on_off} {
    global ac_track_color ac_active_color
    if {$on_off} {
	$w itemconfig $tnum.strtop -fill $ac_active_color
    } else {
	$w itemconfig $tnum.strtop -fill $ac_track_color
    }
}

proc config_uncoupler {w tnum addr fun} {
    global turnout
    $w bind $tnum <1> "do_acc %W $tnum $addr $fun 1"
    $w bind $tnum <ButtonRelease-1> "do_acc %W $tnum $addr $fun 0"
    set turnout($tnum) $fun
    update_uncoupler_display $w $tnum 0
}

proc do_acc {w tnum addr fun on_off} {
    global turnout
    set turnout($tnum) $fun
    update_uncoupler_display $w $tnum $on_off
    operate_turnout $tnum $addr $on_off
}

# mk_str $w.c s1 80 80 .45 100
# mk_curv $w.c c1 80 80 .45 350 .30

# set FD stdout

proc mk_layout {w} {
    global pi offset_x offset_y
    mklcan $w

    source layout1.tcl
}

# set w .layout
# mk_layout $w

